@@ -1,5 +1,5 @@
 package Beam::Emitter;
-our $VERSION = '1.002';
+our $VERSION = '1.003';
 # ABSTRACT: Role for event emitting classes
 
 use strict;
@@ -69,7 +69,8 @@ sub subscribe {
     weaken $self;
     weaken $sub;
     return sub {
-        $self->unsubscribe($name => $sub);
+        $self->unsubscribe($name => $sub)
+	  if defined $self;
     };
 }
 
@@ -125,11 +126,19 @@ sub unsubscribe {
 
 sub emit {
     my ( $self, $name, %args ) = @_;
+
+    return unless exists $self->_listeners->{$name};
+
     my $class = delete $args{ class } || "Beam::Event";
     $args{ emitter  } = $self;
     $args{ name     } = $name;
     my $event = $class->new( %args );
-    for my $listener ( @{ $self->_listeners->{$name} } ) {
+
+    # don't use $self->_listeners->{$name} directly, as callbacks may unsubscribe
+    # from $name, changing the array, and confusing the for loop
+    my @listeners = @{ $self->_listeners->{$name} };
+
+    for my $listener ( @listeners  ) {
         $listener->( $event );
         last if $event->is_stopped;
     }
@@ -148,7 +157,14 @@ sub emit {
 
 sub emit_args {
     my ( $self, $name, @args ) = @_;
-    for my $listener( @{ $self->_listeners->{$name} } ) {
+
+    return unless exists $self->_listeners->{$name};
+
+    # don't use $self->_listeners->{$name} directly, as callbacks may unsubscribe
+    # from $name, changing the array, and confusing the for loop
+    my @listeners = @{ $self->_listeners->{$name} };
+
+    for my $listener ( @listeners ) {
         $listener->( @args );
     }
     return;
@@ -166,7 +182,7 @@ Beam::Emitter - Role for event emitting classes
 
 =head1 VERSION
 
-version 1.002
+version 1.003
 
 =head1 SYNOPSIS
 
@@ -416,12 +432,16 @@ Doug Bell <preaction@cpan.org>
 
 =head1 CONTRIBUTORS
 
-=for stopwords Joel Berger Yanick Champoux
+=for stopwords Diab Jerius Joel Berger Yanick Champoux
 
 =over 4
 
 =item *
 
+Diab Jerius <djerius@cfa.harvard.edu>
+
+=item *
+
 Joel Berger <joel.a.berger@gmail.com>
 
 =item *
